
/**
 * \file CanBasicDemo.c
 * \brief Demo CanBasicDemo
 *
 * \version iLLD_Demos_1_0_1_10_0
 * \copyright Copyright (c) 2014 Infineon Technologies AG. All rights reserved.
 *
 *
 *                                 IMPORTANT NOTICE
 *
 *
 * Infineon Technologies AG (Infineon) is supplying this file for use
 * exclusively with Infineon's microcontroller products. This file can be freely
 * distributed within development tools that are supporting such microcontroller
 * products.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 */

/******************************************************************************/
/*----------------------------------Includes----------------------------------*/
/******************************************************************************/

#include <stdio.h>
#include "CanBasicDemo.h"
#include <Cpu/Irq/IfxCpu_Irq.h>
#include "IfxCpu.h"
#include "IfxStm.h"
/******************************************************************************/
/*-----------------------------------Macros-----------------------------------*/
/******************************************************************************/
#define IFX_INTTOS_CAN_TXRX 0
#define CAN_TX_DELAY_MS     100u
/******************************************************************************/
/*--------------------------------Enumerations--------------------------------*/
/******************************************************************************/

/******************************************************************************/
/*-----------------------------Data Structures--------------------------------*/
/******************************************************************************/

/******************************************************************************/
/*------------------------------Global variables------------------------------*/
/******************************************************************************/

App_CanBasic g_CanBasic;

/******************************************************************************/
/*-------------------------Function Prototypes--------------------------------*/
/******************************************************************************/

/******************************************************************************/
/*------------------------Private Variables/Constants-------------------------*/
/******************************************************************************/
static volatile uint32 numTxmsgs[IFXCAN_NUM_MODULES];
static volatile uint32 numRxmsgs[IFXCAN_NUM_MODULES];

/******************************************************************************/
/*-------------------------Function Implementations---------------------------*/
/******************************************************************************/

/** \addtogroup IfxLld_Demo_CanDemo_SrcDoc_Main_Interrupt
 * \{ */

/** \} */

/** \brief Handle Tx interrupt on CAN node0 module 0.
 *
 * \isrProvider \ref IFX_INTTOS_CAN_TXRX
 * \isrPriority \ref ISR_PRIORITY_CAN1_TX
 *
 */
IFX_INTERRUPT(Tx_Interrput1_ISR, IFX_INTTOS_CAN_TXRX, ISR_PRIORITY_CAN1_TX)
{
	Ifx_CAN   *canSfr = &MODULE_CAN0;

	if(CAN0_IR0.B.TC == 1)
	{
		Ifx_CAN_N *nodeSfr = IfxCan_getNodePointer(canSfr,(IfxCan_NodeId)0);
		IfxCan_Node_clearInterruptFlag(nodeSfr,IfxCan_Interrupt_transmissionCompleted);
		numTxmsgs[1]++;
	}
}


/** \brief Handle Rx interrupt on CAN node0 module 0.
 *
 * \isrProvider \ref IFX_INTTOS_CAN_TXRX
 * \isrPriority \ref ISR_PRIORITY_CAN0_RX
 *
 */
IFX_INTERRUPT(Rx_Interrput0_ISR, IFX_INTTOS_CAN_TXRX, ISR_PRIORITY_CAN0_RX)
{
   Ifx_CAN   *canSfr =  &MODULE_CAN0;

   if(CAN0_IR0.B.DRX == 1)
   {
      Ifx_CAN_N *nodeSfr = IfxCan_getNodePointer(canSfr,(IfxCan_NodeId)0);
      IfxCan_Node_clearInterruptFlag(nodeSfr,IfxCan_Interrupt_messageStoredToDedicatedRxBuffer);
      numRxmsgs[0]++;
   }

}



/** \brief Demo init API
 *
 * This function is called from main during initialization phase
 * demo waits for a rx message with ID 0x100 and reply with id 0x200
 * reply data is teh rx date from ID 0x100.
 */
void CanBasicDemo_init(void)
{
   IfxScuCcu_Config IfxScuCcuConfig;

   /* interrupt groups configuration */
   volatile Ifx_SRC_SRCR *srcPointer;
   /* disable interrupts */
   boolean interruptState = IfxCpu_disableInterrupts();

   /* Default clock initialisation */
   IfxScuCcu_initConfig(&IfxScuCcuConfig);
   IfxScuCcu_init(&IfxScuCcuConfig);

   /* CAN0 initilaisation */
      {
         /*create module configuration*/
         IfxCan_Can_Config canConfig;
         IfxCan_Can_initModuleConfig(&canConfig, &MODULE_CAN0);

         /*initialize module*/
         IfxCan_Can_initModule(&g_CanBasic.drivers.can[0], &canConfig);
      }

   /*create node configuration*/

   /*CAN 0 Node0 as receive node,*/
      {
         IfxCan_Can_NodeConfig nodeConfig;
         IfxCan_Can_initNodeConfig(&nodeConfig, &g_CanBasic.drivers.can[0]);

         nodeConfig.nodeId =   IfxCan_NodeId_0;
         nodeConfig.baudRate   = 500000;
         nodeConfig.frame.type = IfxCan_FrameType_transmitAndReceive;

         nodeConfig.messageRAM.standardFilterListStartAddress = 0x100;
         nodeConfig.messageRAM.rxBuffersStartAddress          = 0x200;
         nodeConfig.messageRAM.txBuffersStartAddress          = 0x400;
         nodeConfig.messageRAM.baseAddress                    = (uint32) &MODULE_CAN0;

         IfxCan_Can_Pins pins;
         pins.rxPin = &IfxCan_RXD00B_P20_7_IN;
         pins.rxPinMode = IfxPort_InputMode_pullUp;
         pins.txPin = &IfxCan_TXD00_P20_8_OUT;
         pins.txPinMode = IfxPort_OutputMode_pushPull;
         pins.padDriver = IfxPort_PadDriver_cmosAutomotiveSpeed2;
         nodeConfig.pins = &pins;

         /* For interrupt configuration user needs to configure the priority and select one of the 16 interrupt lines*/
         nodeConfig.interruptConfig.reint.priority = ISR_PRIORITY_CAN0_RX;
         nodeConfig.interruptConfig.reint.interruptLine = IfxCan_InterruptLine_14;
         nodeConfig.interruptConfig.reint.typeOfService = IfxCpu_Irq_getTos(IfxCpu_getCoreIndex());

         /*Enabling interrupts*/
         nodeConfig.interruptConfig.traco.priority = ISR_PRIORITY_CAN1_TX;
         nodeConfig.interruptConfig.traco.interruptLine = IfxCan_InterruptLine_15;
         nodeConfig.interruptConfig.traco.typeOfService = IfxCpu_Irq_getTos(IfxCpu_getCoreIndex());

         IfxCan_Can_initNode(&g_CanBasic.drivers.canNode[0], &nodeConfig);

         Ifx_CAN_N *nodeSfr = IfxCan_getNodePointer(g_CanBasic.drivers.canNode[0].can, nodeConfig.nodeId);

         IfxCan_Node_enableConfigurationChange(nodeSfr);
         /* Enable interrupts in CAN */
         IfxCan_Node_enableInterrupt(nodeSfr, IfxCan_Interrupt_messageStoredToDedicatedRxBuffer);
         IfxCan_Node_disableConfigurationChange(nodeSfr);
      }

   // set filters
      {
         // Initialise the filter strcture
         IfxCan_Filter filter;

         /* for dedicated buffers id2 is ignored, so id is unique*/
         // set filter0 for rxBuffer 0
         filter.number = 0;
         filter.elementConfiguration = IfxCan_FilterElementConfiguration_storeInRxBuffer;
         filter.id1 = 0x100;

         filter.rxBufferOffset = IfxCan_RxBufferId_0;

         IfxCan_Can_setStandardFilter(&g_CanBasic.drivers.canNode[0], &filter);

      }

   /* enable interrupts again */
   IfxCpu_restoreInterrupts(interruptState);


}


/** \brief Demo run API
 *
 * This function is called from main, background loop
 */
void CanBasicDemo_run(void)
{
   uint32 rxData[2];

   /* Initialise the message strcture */
   IfxCan_Message txMsg;
   IfxCan_Can_initMessage(&txMsg);

   txMsg.messageId = 0x200;
   txMsg.bufferNumber = 0;

   /*Initialise the message structure with dummy values, will be replaced by the received values*/
   IfxCan_Message rxMsg;
   IfxCan_Can_initMessage(&rxMsg);

   /*specify the rx buffer number where the data will be received*/
   rxMsg.bufferNumber = 0;

   while ((IfxCan_Can_isNewDataReceived(&g_CanBasic.drivers.canNode[0], (IfxCan_RxBufferId) rxMsg.bufferNumber)) == 0)
      {};

   /*read message*/
   IfxCan_Can_readMessage(&g_CanBasic.drivers.canNode[0], &rxMsg, rxData);
   /*transmit what we just read*/
   while (IfxCan_Can_sendMessage(&g_CanBasic.drivers.canNode[0], &txMsg, rxData) == IfxCan_Status_notSentBusy)
      {};

}

